/*
   LCD Keypad with 3 buttons and SD-Card
   Created by Lorenzo Cervelli - 2023
*/

// Font 'Cuore' per display
byte heart[8] = {
  0b00000,
  0b01010,
  0b11111,
  0b11111,
  0b11111,
  0b01110,
  0b00100,
  0b00000,
};

// Per la navigazione nel menu`
int RigaMenu = 0;
int ColonnaMenu = 0;
String RigaSelez = "";

// Impostazione display
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2);


void SetupModule()
{
    Serial.begin(115200);
    
    // Inizializzazione display
    lcd.init();
    lcd.backlight();
    lcd.createChar(3, heart);
    lcd.setCursor(2, 0); // Col, Row
    lcd.print("I \x03 Arduino");
    lcd.setCursor(1, 0); // Col, Row
    lcd.blink();

    // Valore iniziale da mostrare sul display
    if (MenuItems[RigaMenu][ColonnaMenu] != "")
    {   
        // Riga 1
        lcd.setCursor(0, 1); // Col, Row
        lcd.print(MenuItems[RigaMenu][ColonnaMenu]);
    } 
}


// La seguente funzione estrae i caratteri presenti in `src` tra `m` e `n` (escluso `n`)
char* substr(const char *src, int m, int n)
{
    // ottiene la lunghezza della stringa di destinazione
    int len = n - m;
 
    // alloca (len + 1) caratteri per la destinazione (+1 per carattere nullo extra)
    char *dest = (char*)malloc(sizeof(char) * (len + 1));
 
    // estrae i caratteri tra il m'esimo e l'nesimo indice dalla stringa di origine
    // e copiali nella stringa di destinazione
    for (int i = m; i < n && (*(src + i) != '\0'); i++)
    {
        *dest = *(src + i);
        dest++;
    }
 
    // termina con null la stringa di destinazione
    *dest = '\0';
 
    // restituisce la stringa di destinazione
    return dest - len;
}


void AggiornaDisplay()
{
    lcd.clear();
    lcd.setCursor(0, 1);    // Riga 1 (Col, Row)

    if (SelectedItems[RigaMenu][ColonnaMenu] == true)
    {
        // Concatenazione char array: "MenuItems[RigaMenu][ColonnaMenu]" + "(*)"
        char* A = MenuItems[RigaMenu][ColonnaMenu];
        int L=strlen(A);
        char B[L];
        strcpy( B, A );
        strcat( B, "(*)" );
        String myString = String(B);
                                                
        // Aggiorno il display mostrando solo l'ultima parte della stringa del menu
        int LastRec = myString.lastIndexOf("@");                     
        if (LastRec>0)
        {
            String result = myString.substring((LastRec + 2), myString.length());
            lcd.print(result);  
        }
        else
        {
            lcd.print(myString);
        } 
    }
    else
    {
        String myString = String(MenuItems[RigaMenu][ColonnaMenu]);
                                                
        // Aggiorno il display mostrando solo l'ultima parte della stringa del menu
        int LastRec = myString.lastIndexOf("@");                     
        if (LastRec>0)
        {
            String result = myString.substring((LastRec + 2), myString.length());
            lcd.print(result);
        }
        else
        {
            lcd.print(myString);
        } 
    }

    lcd.setCursor(2, 0); // Col, Row
    lcd.print("I \x03 Arduino");
}


void Avanti()
{
    Begin_Avanti:
    
    String ValA = MenuItems[RigaMenu][ColonnaMenu];      
    String ValB = "";
    int numRows = sizeof(MenuItems)/sizeof(MenuItems[0]);
                   
    if (ColonnaMenu > 0)
    {
        do
        {
            // Righe
            if (RigaMenu < numRows - 1)
            {
                RigaMenu++;
            }
            else
            {
                RigaMenu = 0;
            }

            ValB = MenuItems[RigaMenu][ColonnaMenu];
        }
        while ( ValA == ValB);

        // Verifico che la riga selezionata contenga
        String ValAfinal = MenuItems[RigaMenu][ColonnaMenu];    
        int L = RigaSelez.length();
        String subs = ValAfinal.substring(0, L);

        if (RigaSelez!=subs)
        {
            goto Begin_Avanti;
        }
    }
    else
    {
        do
        {
            // Righe
            if (RigaMenu < numRows - 1)
            {
                RigaMenu++;
            }
            else
            {
                RigaMenu = 0;
            }

            ValB = MenuItems[RigaMenu][ColonnaMenu];
        }
        while (ValA == ValB);

        if (ValB=="")
        {
            goto Begin_Avanti;
        }
    }

    AggiornaDisplay();
}


void Indietro()
{
    Begin_Indietro:

    String ValA = MenuItems[RigaMenu][ColonnaMenu];
    String ValB = "";  
    int numRows = sizeof(MenuItems)/sizeof(MenuItems[0]);

    if (ColonnaMenu > 0)
    {
        do
        {
            // Righe
            if (RigaMenu > 0)
            {
                RigaMenu--;
            }
            else
            {
                RigaMenu = numRows - 1;
            }

            ValB = MenuItems[RigaMenu][ColonnaMenu];
        }
        while (ValA == ValB);

        // Verifico che la riga selezionata contenga
        String ValAfinal = MenuItems[RigaMenu][ColonnaMenu];    
        int L = RigaSelez.length();
        String subs = ValAfinal.substring(0, L);

        if (RigaSelez!=subs)
        {
            goto Begin_Indietro
    ;
        }
    }
    else
    {
        do
        {
            // Righe
            if (RigaMenu > 0)
            {
                RigaMenu--;
            }
            else
            {
                RigaMenu = numRows - 1;
            }

            ValB = MenuItems[RigaMenu][ColonnaMenu];
        }
        while (ValA == ValB);

        if (ValB=="")
        {
            goto Begin_Indietro
    ;
        }
    }   

    AggiornaDisplay();
}

String Seleziona()
{
    // Riga selezionata
    RigaSelez = MenuItems[RigaMenu][ColonnaMenu];
    int numRows = sizeof(MenuItems)/sizeof(MenuItems[0]);
    int numCols = sizeof(MenuItems[0])/sizeof(MenuItems[0][0]);
    String Rtr = "";

    if (ColonnaMenu <= numCols - 1)
    {
        String vA = MenuItems[RigaMenu][ColonnaMenu];
        String Vsubs = vA.substring(vA.length() - 4, vA.length());

        if (Vsubs != "Back")
        {
            // Stringa di ritorno            
            Rtr =  vA + ";" + RigaMenu + ";" + ColonnaMenu + ";";

            // Mostro solo l'ultima parte della stringa del menu
            int LastRec = vA.lastIndexOf('@');
            if (LastRec > 0)
            {
                String result = vA.substring((LastRec + 2), vA.length());
                
                // Stringa di ritorno            
                //Rtr =  result + ";" + RigaMenu + ";" + ColonnaMenu + ";";
            }
            else
            {
                // Stringa di ritorno 
                //Rtr =  vA + ";" + RigaMenu + ";" + ColonnaMenu + ";";
            }

            AggiornaDisplay();
        }
    }
   
    // Back to previous brach
    if (ColonnaMenu > 0)
    {
        char* A = MenuItems[RigaMenu][ColonnaMenu];
        int L = strlen(A);
        char* subs = substr(A, L - 4, L);  // Substring char-array
        char* test = "Back";
            
        if( strcmp(subs, test) == 0)
        {
            ColonnaMenu--;

            if (ColonnaMenu > 0)
            {
                RigaSelez = MenuItems[RigaMenu][ColonnaMenu - 1];
            }
                       
            AggiornaDisplay();
            return Rtr;
        }
    }

    // Drill-Down
    if (ColonnaMenu < numCols - 1)
    {
        if (MenuItems[RigaMenu][ColonnaMenu + 1] == "")
        {
            if (ColonnaMenu > 0)
            {
                RigaSelez = MenuItems[RigaMenu][ColonnaMenu - 1];
            }
        }
        else
        {
            ColonnaMenu++;           
            
            AggiornaDisplay();
            return Rtr;
        }
    }
    else
    {
        RigaSelez = MenuItems[RigaMenu][ColonnaMenu-1];      
        return Rtr;
    }

    return Rtr;
}


